@@ -21,306 +21,70 @@
    >>> loop.run_until_complete(Render.ReneringManage())
    >>> Render.ReprenderIMG.show()
 
-
-
 BUG emoji不被字符计数，导致偏差。（可能占用字符数目不一样）
 
 copyright: (c) 2021 by NGWORKS.
 
 license: MIT.
 """
+# 数据验证
+from .format import DynamicCard,Dynamic,Display
+# 初始化
+from .initialize import bsepth,muniMap,euniMap,cuniMap,workpath
+from .initialize import NotoColorEmoji,NotoSansCJK,CODE2000,Unifont,fontList
+# 文字工具 
+from .textTools import get_font_render_size,KeyWordsCut,AoutLine
+
 import os,aiohttp,asyncio, time,re, qrcode
-from typing import Optional, Union, List
-from pydantic import BaseModel, Json
 from PIL import Image, ImageFont, ImageDraw
 from urllib.parse import urlparse
 from io import BytesIO
-from fontTools.ttLib.ttFont import TTFont
-from matplotlib.font_manager import fontManager
-from pathlib import Path
+# 静态资源图片
+faceMark = Image.open(bsepth + 'element/hm.png')
+userauth = Image.open(bsepth + 'element/user-auth.png')
 
-workpath = os.getcwd()
-bsepth = os.path.dirname(__file__) + '/'
+def set_tmp(tmp = None):
+    if tmp is None:
+        path = './tmp'
+    else:
+        path = tmp
 
-NotoSansCJK = bsepth + r'typeface/NotoSansCJKsc-Regular.otf'
-NotoColorEmoji = bsepth + r'typeface/NotoColorEmoji.ttf'
-CODE2000 = bsepth + r'typeface/CODE2000.ttf'
-Unifont = bsepth + r'typeface/Unifont.ttf.ttf'
-arial = bsepth + r'typeface/reserve/arial.ttf'
-himalaya = bsepth + r'typeface/reserve/himalaya.ttf'
-
-
-rtime = []
-mfont = TTFont(NotoSansCJK)
-muniMap = mfont['cmap'].tables[0].ttFont.getBestCmap()
-efont = TTFont(NotoColorEmoji)
-euniMap = efont['cmap'].tables[0].ttFont.getBestCmap()
-Cfont = TTFont(CODE2000)
-cuniMap = Cfont['cmap'].tables[0].ttFont.getBestCmap()
-
-tfl = [[f.fname, f.name, f.style] for f in fontManager.ttflist]
-f = fontManager.ttflist
-okf = []
-
-for f in tfl:
-    f_path = Path(f[0])
-    try:
-        if f_path.suffix not in ['.ttf', '.TTF', '.otf', '.OTF']:
-            print(f'字体{f[0]}不符合样式需求，已经排除。')
-            continue
-        oofont = TTFont(f[0])
-        oouniMap = oofont['cmap'].tables[0].ttFont.getBestCmap()
-        okf.append([oouniMap, f[0]])
-    except:
-        print(f'导入{f[0]}失败，无需处理')
-
-# 加入后备字体
-okf.append([TTFont(arial)['cmap'].tables[0].ttFont.getBestCmap(), arial])
-okf.append([TTFont(himalaya)['cmap'].tables[0].ttFont.getBestCmap(), himalaya])
-
-
-
-class info(BaseModel):
-    """用户信息"""
-    uid: Optional[int] = None
-    uname: Optional[str] = None
-    face: Optional[str] = None
-    head_url: Optional[str] = None
-    name: Optional[str] = None
-
-
-class level_info(BaseModel):
-    """等级"""
-    current_level: int
-
-
-class pendant(BaseModel):
-    """挂件"""
-    pid: int
-    name: str
-    image: str
-
-
-class official_verify(BaseModel):
-    """认证用户情况"""
-    type: int
-    desc: str
-
-
-class inofcard(BaseModel):
-    official_verify: official_verify
-
-
-class vip(BaseModel):
-    """大会员情况"""
-    vipType: int
-    nickname_color: str
-
-
-class user_profile(BaseModel):
-    info: info
-    level_info: Optional[level_info]
-    pendant: Optional[pendant]
-    card: Optional[inofcard]
-    vip: Optional[vip]
-
-
-class desc(BaseModel):
-    type: int
-    timestamp: int
-    view: int
-    orig_dy_id: Optional[int] = None
-    orig_type: int
-    user_profile: user_profile
-    dynamic_id: int
-
-
-class topic_details(BaseModel):
-    topic_name: str
-    is_activity: bool
-
-# emoji 信息
-
-
-class emoji_details(BaseModel):
-    emoji_name: str
-    id: int
-    text: str
-    url: str
-
-
-class Topic_info(BaseModel):
-    topic_details: List[topic_details]
-
-
-class Emoji_details(BaseModel):
-    emoji_details: List[emoji_details]
-
-
-class at_control(BaseModel):
-    data: int
-    length: int
-    location: int
-    type: int
-
-
-class pictures(BaseModel):
-    img_src: str
-    img_height: int
-    img_width: int
-
-
-class DynamicItem(BaseModel):
-    at_control: Optional[Union[Json, str]] = None
-    description: Optional[str] = None
-    upload_time: Optional[int] = None
-    content: Optional[str] = None
-    ctrl: Optional[Union[Json, str]] = None
-    pictures: Optional[Union[str, List[pictures]]]
-
-
-class vest(BaseModel):
-    content: str
-
-
-class apiSeasonInfo(BaseModel):
-    title: Optional[str]
-    type_name: str
-
-
-class Dynamic(BaseModel):
-    item: Optional[DynamicItem] = None
-    dynamic: Optional[str] = None
-    pic: Optional[str] = None
-    title: Optional[str] = None
-    origin: Optional[Json] = None
-    image_urls: Optional[List] = None
-    summary: Optional[str] = None
-    vest: Optional[vest]
-    origin_user: Optional[user_profile] = None
-
-    duration: Optional[int] = None
-
-    user: Optional[info]
-    owner: Optional[info]
-    author: Optional[info]
-
-    cover: Optional[str] = None
-    area_v2_name: Optional[str] = None
-
-    apiSeasonInfo: Optional[apiSeasonInfo]
-
-    new_desc: Optional[str] = None
-
-
-class desc_first(BaseModel):
-    text: str
-
-
-class reserve_attach_card(BaseModel):
-    title: str
-    desc_first: Optional[Union[str, desc_first]]
-    desc_second: Optional[str]
-    cover_url: Optional[str]
-    head_text: Optional[str]
-
-
-class add_on_card_info(BaseModel):
-    add_on_card_show_type: int
-    reserve_attach_card: Optional[reserve_attach_card]
-    vote_card: Optional[Json]
-    attach_card: Optional[reserve_attach_card]
-
-
-class Display(BaseModel):
-    topic_info: Optional[Topic_info] = None
-    emoji_info: Optional[Emoji_details] = None
-    add_on_card_info: Optional[List[add_on_card_info]]
-    origin: Optional[dict]
-
-
-class DynamicCard(BaseModel):
-    """
-    详细信息的card
-        :desc 动态基本信息
-        :card 动态的内容
-    """
-    desc: Optional[desc]
-    card: Json[Dynamic]
-    display: Display
-
-
-class DynamicData(BaseModel):
-    """动态详细信息data"""
-    card: DynamicCard
-
-
-class DynamicDetail(BaseModel):
-    """动态详细信息接口返回"""
-    code: int
-    data: DynamicData
+    if os.path.isabs(path):
+        tmp = path 
+    else:
+        os.chdir(workpath)
+        tmp = os.path.abspath(path)
+    tmp = tmp + '/'
+    tmp_path = tmp
+    pathlist = [tmp_path + 'face/',tmp_path + 'pendant/',tmp_path + 'emoji/']
+    for p in pathlist:
+        print('r1',p)
+        if not os.path.isdir(p):
+            print('r')
+            os.makedirs(p)
+    return tmp_path
 
 
 class DynamicPictureRendering:
     """动态渲染器
        --------
     """
-
-    def __init__(self, Dynamicdata, tmp_path = None ):
-        self.DynamicImg = {}
+    def __init__(self,path = None):
         self.HeadImg = []
         self.EmojiImg = []
-        self.FunctionBlockImg = None
-        self.headRenderingImg = None
-        self.NGSSTrckerImg = None
-        self.b23 = None
-        self.DynamicData = DynamicCard(**Dynamicdata)
-        self.DynamicId = self.DynamicData.desc.dynamic_id
-        self.tmp_path  =  tmp_path
+        self.DynamicData = None
+        self.DynamicId = None
+        self.tmp_path  =  set_tmp(path)
 
-    def set_tmp_path(self):
-            if self.tmp_path is None:
-                path = './tmp'
-            else:
-                path = self.tmp_path
-
-            if os.path.isabs(path):
-                print('1111111')
-                self.tmp_path = path 
-            else:
-                os.chdir(workpath)
-                self.tmp_path = os.path.abspath(path)
-            self.tmp_path = self.tmp_path + '/'
-            pathlist = [self.tmp_path + 'face/',self.tmp_path + 'pendant/',self.tmp_path + 'emoji/']
-            for p in pathlist:
-                if not os.path.isdir(p):
-                    os.makedirs(p)
-
-    async def fetch(self, session, url, jsont=True):
+    async def fetch(self, session, url):
         """实现GET请求"""
         async with session.get(url) as response:
-            if jsont:
-                data = await response.json()
-                self.DynamicData = data
-                return data
-            else:
-                return await response.read()
-
-    async def post(self, session, url, data):
-        """实现POST请求"""
-        async with session.post(url, data=data) as response:
-            data = await response.json()
-            self.b23 = data['data']['content']
-
-    async def load(self, session, url, jsont=True):
-        """实现下载图片"""
-        async with session.get(url) as response:
-            return await response.json()
+            return await response.read()
 
     async def getPage(self, url, count=0, tp=0, sz=0):
         """下载图片"""
         async with aiohttp.ClientSession() as session:
-            data = await self.fetch(session, url, jsont=False)
+            data = await self.fetch(session, url)
             pic = Image.open(BytesIO(data))
             if tp == 0:
                 return pic
@@ -330,17 +94,8 @@ class DynamicPictureRendering:
                 self.EmojiImg.append({"data": pic, "path": count, "id": sz})
             return pic
 
-    def get_font_render_size(self, FOUNT, fontsize, text):
-        """获取字符精准宽度"""
-        try:
-            font = ImageFont.truetype(FOUNT, fontsize, 0)
-            width, height = font.getsize(text)
-            return (width, height)
-        except:
-            print(f'字符{text}宽度计算失败，已使用默认值')
-            return (fontsize, fontsize)
-
     async def makeQRcode(self, data):
+        """制作二维码"""
         qr = qrcode.QRCode(
             version=1,
             error_correction=qrcode.constants.ERROR_CORRECT_L,
@@ -352,43 +107,6 @@ class DynamicPictureRendering:
         qr.make(fit=True)
         return qr.make_image()
 
-    def AoutLine(self, limt, text, size, color='#000000', fount=NotoSansCJK, x=15, y=0):
-        """非富文本样式计算"""
-        START_X = x
-        START_Y = y
-        SZ = 0
-        FOUNT_SIZE = size
-        LINE_HIGHT = 15
-        LINE_LIMT = limt
-        pl = []
-        for i in text:
-            if START_X >= LINE_LIMT:
-                START_X = x
-                START_Y += FOUNT_SIZE + LINE_HIGHT
-            wihdt = self.get_font_render_size(fount, FOUNT_SIZE, i)[0]
-            pl.append({"t": i, "d": (START_X, START_Y), "c": color, "f": fount})
-            START_X += wihdt
-        return pl
-
-    def KeyWordsCut(self, KeyWord, paragraph):
-        """字符串位置检测"""
-        keywordslen = len(KeyWord)
-        wordsList = []
-        i = 0
-        while True:
-            l = paragraph.find(KeyWord)
-            if l == -1:
-                break
-            paragraph = paragraph[l+keywordslen:]
-            if len(wordsList) != 0:
-                b = wordsList[i]
-                l = b + l + keywordslen
-                i += 1
-            wordsList.append(l)
-        wordsList = list(set(wordsList))
-        wordsList.sort()
-        return wordsList
-
     async def headRendering(self, desc):
         """
         头部信息渲染器：
@@ -441,11 +159,8 @@ class DynamicPictureRendering:
         HeadRender = Image.new("RGB", (cpx, cpx), "#FFFFFF")
         if face == None:
             face = Image.open(facePath).resize((fpx, fpx), Image.ANTIALIAS)
-        facem = Image.open(
-            bsepth + 'element/hm.png').resize((fpx, fpx), Image.ANTIALIAS)
-        HeadRender.paste(
-            face, (int((cpx-fpx)/2), int((cpx-fpx)/2)), mask=facem)
-        facem.close()
+        facem = faceMark.resize((fpx, fpx), Image.ANTIALIAS)
+        HeadRender.paste(face, (int((cpx-fpx)/2), int((cpx-fpx)/2)), mask=facem)
 
         if pendant.image != "":
             """渲染头像框"""
@@ -456,20 +171,18 @@ class DynamicPictureRendering:
             HeadRender.paste(pendant, (int((cpx-ppx)/2),
                              int((cpx-ppx)/2)), mask=pendant)
 
-        if official.type != -1 or vip.vipType != 1:
+        if official.type != -1 or vip.vipType != 0:
             """渲染小闪电和大会员标"""
-            if vip.vipType != 1:
+            if vip.vipType != 0:
                 box = (69, 16, 94, 41)
             if official.type == 0:
                 box = (35, 16, 60, 41)
             elif official.type == 1:
                 box = (1, 16, 26, 41)
-            officialimg = Image.open(bsepth + 'element/user-auth.png')
-            officialimg = officialimg.crop(box).resize(
+            officialimg = userauth.crop(box).resize(
                 (ico, ico), Image.ANTIALIAS).convert('RGBA')
             HeadRender.paste(officialimg, (fpx+int(ico/2),
                              fpx+int(ico/2)), mask=officialimg)
-            officialimg.close()
         # 名字颜色
         if vip.nickname_color == "":
             nickname_color = (34, 34, 34)
@@ -513,7 +226,7 @@ class DynamicPictureRendering:
         根据数据中的信息生成`样式描述信息`(NGSS)与渲染动态文字照片：
 
         :param data: 动态的数据。
-        :returns:  输出为一个渲染好的`图片`。
+        :returns:  输出为一个渲染好的`PIL图片`。
 
         这个函数可以自动的识别、分割出来文字中的艾特、投票、抽奖、标签功能、同时可以渲染表情包，生成指导渲染的信息：
 
@@ -578,7 +291,7 @@ class DynamicPictureRendering:
                 """表情包分割方案器"""
                 emojiName = emoji.emoji_name
                 wlen = len(emojiName)
-                worldstarList = self.KeyWordsCut(emojiName, Text)
+                worldstarList = KeyWordsCut(emojiName, Text)
                 for w in worldstarList:
                     msg = {"start": w, "end": w+wlen, "len": wlen,
                            "type": 0, "data": {"url": emoji.url, "id": emoji.id}}
@@ -589,7 +302,7 @@ class DynamicPictureRendering:
                 """话题 活动 分割方案器"""
                 topicTag = f'#{topic.topic_name}#'
                 taglen = len(topicTag)
-                worldstarList = self.KeyWordsCut(topicTag, Text)
+                worldstarList = KeyWordsCut(topicTag, Text)
                 for w in worldstarList:
                     msg = {"start": w, "end": w+taglen,
                            "len": taglen, "type": 1, "data": None}
@@ -597,7 +310,7 @@ class DynamicPictureRendering:
 
         # 识别超链接
 
-        reg = r'https?:[0-9a-zA-Z.?&=_@(-/\d]+'
+        reg = r'https?:[0-9a-zA-Z.?#&=_@(-/\d]+'
         url = re.findall(reg, Text)
         url = list(set(url))
         if len(url) != 0:
@@ -606,7 +319,7 @@ class DynamicPictureRendering:
                 reg = r'(.*).bilibili.com'
                 if re.match(reg, n) or n == 'b23.tv':
                     urllen = len(i)
-                    worldstarList = self.KeyWordsCut(i, Text)
+                    worldstarList = KeyWordsCut(i, Text)
                     for w in worldstarList:
                         msg = {"start": w, "end": w+urllen, "len": urllen,
                                "type": 2, "data": {"control": 5}}
@@ -676,6 +389,7 @@ class DynamicPictureRendering:
                 if len(text.split('\n')) > 1:
                     element['text'] = text.split('\n')
         del type, text, element
+        
         # RenderList 最终样式规定器 计算宽度，分割，细化最初的NGSS
         START_X, START_Y = (0, 0)
         SZ = 1
@@ -700,7 +414,6 @@ class DynamicPictureRendering:
                     {"id": -1, "d": (START_X, START_Y), "u": element['data']})
                 START_X += FOUNT_SIZE + SZ + 8
             else:
-                # 如果不是list 就没有换行
                 if isinstance(text, list):
                     pp = 1
                     for t in text:
@@ -713,25 +426,25 @@ class DynamicPictureRendering:
 
                             if ord(s) in muniMap.keys():
                                 f = MainFontPath
-                                wihdt = self.get_font_render_size(
+                                wihdt = get_font_render_size(
                                     f, FOUNT_SIZE, s)[0]
                             elif ord(s) in euniMap.keys():
                                 f = EmojiFontPath
                                 wihdt = FOUNT_SIZE
                             elif ord(s) in cuniMap.keys():
                                 f = CODE2000
-                                wihdt = self.get_font_render_size(
+                                wihdt = get_font_render_size(
                                     f, FOUNT_SIZE, s)[0]
                             else:
                                 f = Unifont
-                                for ppp in okf:
+                                for ppp in fontList:
                                     try:
                                         if ord(s) in ppp[0].keys():
                                             f = ppp[1]
                                             break
                                     except:
                                         pass
-                                wihdt = self.get_font_render_size(
+                                wihdt = get_font_render_size(
                                     f, FOUNT_SIZE, s)[0]
                             rl.append(
                                 {"t": s, "d": (START_X, START_Y), "c": FountColor, "f": f})
@@ -755,18 +468,18 @@ class DynamicPictureRendering:
 
                         if ord(i) in muniMap.keys():
                             f = MainFontPath
-                            wihdt = self.get_font_render_size(
+                            wihdt = get_font_render_size(
                                 f, FOUNT_SIZE, i)[0]
                         elif ord(i) in euniMap.keys():
                             f = EmojiFontPath
                             wihdt = FOUNT_SIZE
                         elif ord(i) in cuniMap.keys():
                             f = CODE2000
-                            wihdt = self.get_font_render_size(
+                            wihdt = get_font_render_size(
                                 f, FOUNT_SIZE, i)[0]
                         else:
                             f = Unifont
-                            for ppp in okf:
+                            for ppp in fontList:
                                 try:
                                     if ord(i) in ppp[0].keys():
                                         f = ppp[1]
@@ -774,7 +487,7 @@ class DynamicPictureRendering:
                                 except:
                                     pass
 
-                            wihdt = self.get_font_render_size(
+                            wihdt = get_font_render_size(
                                 f, FOUNT_SIZE, i)[0]
 
                         rl.append(
@@ -865,16 +578,16 @@ class DynamicPictureRendering:
         ttf_path = NotoSansCJK
         nicknameFont = ImageFont.truetype(ttf_path, 25)
         sFont = ImageFont.truetype(ttf_path, 20)
-        # 投稿视频和直播
+        # 投稿视频 直播 电视剧
         if type in [4099, 4308, 8]:
             if type == 4099:
-                pic = card.cover + '@720w.webp'
+                pic = card.cover + '@480w.webp'
                 title = card.new_desc
-            if type == 4308:
-                pic = self.DynamicData.card.origin['live_play_info']['cover']
+            elif type == 4308:
+                pic = self.DynamicData.card.origin['live_play_info']['cover'] + '@480w.webp'
                 title = self.DynamicData.card.origin['live_play_info']['title']
             else:
-                pic = card.pic + '@720w.webp'
+                pic = card.pic + '@480w.webp'
                 title = card.title
 
             pic = await self.getPage(pic)
@@ -913,7 +626,7 @@ class DynamicPictureRendering:
             img_draw = ImageDraw.Draw(img)
             textcount, allwidth = (0, 0)
             for t in title:
-                wihdt = self.get_font_render_size(NotoSansCJK, 25, t)[0]
+                wihdt = get_font_render_size(NotoSansCJK, 25, t)[0]
                 textcount += 1
                 allwidth += wihdt
                 if allwidth > 675:
@@ -929,10 +642,7 @@ class DynamicPictureRendering:
             # 根据图片数量生成渲染规则
             if piccount == 1:
                 lm = 1
-                if pictures[0].img_width < 1900 and pictures[0].img_height < 1900:
-                    w, h = (pictures[0].img_width, pictures[0].img_height)
-                else:
-                    w, h = (518, None)
+                w, h = (518, None)
             elif piccount in [2, 4]:
                 # 当 图片数量 等于2 或等于4时
                 # 图片是两个一排 宽度平均分
@@ -1005,7 +715,7 @@ class DynamicPictureRendering:
                 summary = summary[:52] + '...'
             title = card.title
             # ISDO 计算标题文字高度
-            titlestly = self.AoutLine(
+            titlestly = AoutLine(
                 675, title, 30, color='#000000', fount=NotoSansCJK)
             lasttitle = titlestly[len(titlestly)-1]['d'][1] + 30
             tasks = []
@@ -1036,7 +746,7 @@ class DynamicPictureRendering:
                     w = h
             lastpic = fulren[len(fulren)-1][1][1] + w
             if summary != '':
-                summstly = self.AoutLine(
+                summstly = AoutLine(
                     675, summary, 25, color='#666666', fount=NotoSansCJK, y=lastpic+20)
                 lastsumm = summstly[len(summstly)-1]['d'][1] + 30
             else:
@@ -1141,7 +851,10 @@ class DynamicPictureRendering:
             # 组装信息
             card = card.origin
             display = display.origin
+            if card is None:
+                return None
             card = Dynamic(**card)
+
             if display:
                 display = Display(**display)
             else:
@@ -1188,13 +901,13 @@ class DynamicPictureRendering:
                 img.paste(i[0], i[1])
             return img
 
-    async def ReneringManage(self):
+    async def ReneringManage(self,Dynamicdata):
         """渲染任务管理
            ----------
         """
         # 获取数据
-        self.set_tmp_path()
-        start = time.time()
+        self.DynamicData = DynamicCard(**Dynamicdata)
+        self.DynamicId = self.DynamicData.desc.dynamic_id
         card = self.DynamicData.card
         desc = self.DynamicData.desc
         display = self.DynamicData.display
@@ -1252,6 +965,4 @@ class DynamicPictureRendering:
         img.paste(res[0], (590, h-booten.size[1]+30))
         # img.save('t.jpg')
         # img.save(f'./test/{self.DynamicId}.jpg')
-        end = time.time()
-        rtime.append(end-start)
         self.ReprenderIMG = img
